#!/usr/bin/env python3

import os
import sys
import shutil
import json
import subprocess
import pwd
import random
import string
import glob


PING_USER = 'turku-ping'


def config_get(attribute):
    cmd = ['config-get', attribute]
    return subprocess.check_output(cmd).decode('utf-8').strip()  # IGNORE:E1103


def unit_get(attribute):
    cmd = ['unit-get', attribute]
    return subprocess.check_output(cmd).decode('utf-8').strip()  # IGNORE:E1103


def open_port(port_number, protocol='tcp'):
    cmd = ['open-port', '{}/{}'.format(port_number, protocol)]
    return subprocess.call(cmd)


def juju_log(severity, message):
    cmd = ['juju-log', '--log-level', severity, message]
    subprocess.check_call(cmd)


def json_dump_p(obj, f):
    """Calls json.dump with standard (pretty) formatting"""
    return json.dump(obj, f, sort_keys=True, indent=4, separators=(',', ': '))


def json_dumps_p(obj):
    """Calls json.dumps with standard (pretty) formatting"""
    return json.dumps(obj, sort_keys=True, indent=4, separators=(',', ': '))


def install():
    for f in glob.glob('exec.d/*/charm-pre-install'):
        if os.path.isfile(f) and os.access(f, os.X_OK):
            subprocess.check_call(['sh', '-c', f])

    subprocess.check_call(['apt-get', 'update'])
    subprocess.check_call(
        ['apt-get', '-y', 'install', 'python3', 'python3-setuptools', 'python3-pkg-resources', 'rsync', 'lvm2']
    )

    cwd = os.environ['CHARM_DIR']
    subprocess.check_call(['python3', 'setup.py', 'install'], cwd='%s/files/turku-storage' % cwd)

    try:
        pwd.getpwnam(PING_USER)
    except KeyError:
        subprocess.check_call(['adduser', '--gecos', 'Turku Storage', '--disabled-password', PING_USER])

    shutil.copyfile('files/turku-storage/turku-storage.logrotate',
                    '/etc/logrotate.d/turku-storage')
    shutil.copyfile('files/turku-storage/turku-storage.cron',
                    '/etc/cron.d/turku-storage')
    shutil.copyfile('files/turku-storage.sudo',
                    '/etc/sudoers.d/turku-storage')
    os.chmod('/etc/sudoers.d/turku-storage', 0o440)


def configure():
    for d in ('/etc/turku-storage/config.d',):
        if not os.path.isdir(d):
            os.makedirs(d)
            os.chmod(d, 0o700)

    if os.path.exists('/etc/turku-storage/config.d/50-juju.json'):
        with open('/etc/turku-storage/config.d/50-juju.json') as f:
            config = json.load(f)
    else:
        config = {}

    api_url = config_get('api_url')
    api_auth_name = config_get('api_auth_name')
    api_auth_secret = config_get('api_auth_secret')
    if not (api_url and api_auth_name and api_auth_secret):
        return

    config['api_url'] = api_url
    config['api_auth_name'] = api_auth_name
    config['api_auth_secret'] = api_auth_secret
    # Default to link-dest, but allow upgrade-charm for legacy attic
    # on existing trusty deploys
    if 'snapshot_mode' not in config:
        config['snapshot_mode'] = 'link-dest'
    config['ssh_ping_host'] = unit_get('public-address')
    config['ssh_ping_port'] = 22
    config['ssh_ping_user'] = PING_USER
    config['authorized_keys_command'] = 'sudo /usr/local/bin/turku-storage-ping'

    #if os.path.isdir('/etc/ssh/user-authorized-keys'):
    #    config['authorized_keys_file'] = '/etc/ssh/user-authorized-keys/%s' % PING_USER
    #    config['authorized_keys_user'] = 'root'

    if 'name' not in config:
        config['name'] = os.environ['JUJU_UNIT_NAME']
    if 'secret' not in config:
        config['secret'] = ''.join(random.choice(string.ascii_letters + string.digits) for i in range(30))

    ready_volumes = []
    volumes = {}
    with open('/proc/mounts') as f:
        mounts_text = f.read()
    for v in config_get('volumes').split(','):
        if not v:
            continue
        idpath = None
        realpath = None
        if v.startswith('dir:'):
            dir = v[4:]
        elif os.path.exists('/dev/mapper/%s' % v):
            idpath = '/dev/mapper/%s' % v
            dir = '/media/%s' % v
        elif (len(v) >= 20) and os.path.exists('/dev/disk/by-id/virtio-%s' % v[0:20]):
            idpath = '/dev/disk/by-id/virtio-%s' % v[0:20]
            dir = '/media/%s' % v
        else:
            continue
        if idpath is not None:
            realpath = os.path.realpath(idpath)
        juju_log('INFO', 'Found %s: %s %s %s' % (v, idpath, realpath, dir))
        if not os.path.isdir(dir):
            os.makedirs(dir)
            os.chmod(dir, 0o700)
        ready_volumes.append(dir)
        volumes[v] = {
            'path': dir,
        }
        if (realpath is not None) and (dir not in mounts_text):
            if 'filesystem' not in subprocess.check_output(['file', '-s', '-L', realpath]).decode('utf-8'):
                juju_log('INFO', 'mkfs.ext4 %s' % realpath)
                subprocess.check_call(['mkfs.ext4', realpath])
            juju_log('INFO', 'mount %s %s' % (realpath, dir))
            subprocess.check_call(['mount', realpath, dir])
    config['volumes'] = volumes

    with open('/etc/turku-storage/config.d/50-juju.json', 'w') as f:
        json_dump_p(config, f)

    juju_log('INFO', 'Running turku-storage-update-config')
    subprocess.call(['turku-storage-update-config'])

    open_port(config['ssh_ping_port'])


if __name__ == '__main__':
    hook_name = os.path.basename(sys.argv[0])
    juju_log('INFO', 'Hook: %s' % hook_name)
    if hook_name in ('install', 'upgrade-charm'):
        install()
    elif hook_name == 'config-changed':
        configure()
